@@ -401,8 +401,7 @@ sub restore {
         $conf = join '', <STDIN>;
     }
     else {
-        die "Unable to read $file: $!" unless ( -r $file );
-        open( my $f, '<', $file ) or die $!;
+        open( my $f, '<', $file ) or die "Unable to read $file: $!";
         $conf = join '', <$f>;
         close $f;
         die "Empty or malformed file $file" unless ( $conf =~ /\w/s );
