@@ -8,7 +8,7 @@ use Lemonldap::NG::Handler::Main;
 use Lemonldap::NG::Common::Util qw(getSameSite);
 use URI;
 
-our $VERSION = '2.0.15';
+our $VERSION = '2.19.0';
 
 ## @method hashref tests(hashref conf)
 # Return a hash ref where keys are the names of the tests and values
@@ -33,6 +33,12 @@ sub tests {
 
         # Check if portal is in domain
         portalIsInDomain => sub {
+
+            # Looks like a sub, user knows what they are doing
+            if ( $conf->{portal} =~ /[\$\(&\|"']/ ) {
+                return 1;
+            }
+
             return (
                 1,
                 (
@@ -47,16 +53,20 @@ sub tests {
         portalURL => sub {
             my $url = $conf->{portal};
 
-            # Append or remove trailing slashes
-            $conf->{portal} =~ s%/*$%/%;
-            return (
-                1,
-                (
-                    ( $url =~ m%/$% )
-                    ? ''
-                    : "Portal URL should end with a /"
-                )
-            );
+            # Looks like a sub, user knows what they are doing
+            if ( $url =~ /[\$\(&\|"']/ ) {
+                return 1;
+            }
+            else {
+                return (
+                    1,
+                    (
+                        ( $url =~ m%/$% )
+                        ? ''
+                        : "Portal URL should end with a /"
+                    )
+                );
+            }
         },
 
         # Check if virtual hosts are in the domain
@@ -481,6 +491,20 @@ sub tests {
               );
         },
 
+        samlIssuerNotEnabled => sub {
+            if ( keys %{ $conf->{samlSPMetaDataXML} || {} } ) {
+                if ( $conf->{issuerDBSAMLActivation} ) {
+                    return 1;
+                }
+                else {
+                    return ( 1,
+"SAML service providers require enabling the SAML Issuer in General Parameters"
+                    );
+                }
+            }
+            return 1;
+        },
+
         # Try to parse combination with declared modules
         checkCombinations => sub {
             return 1 unless ( $conf->{authentication} eq 'Combination' );
@@ -711,10 +735,7 @@ sub tests {
         bruteForceProtection => sub {
             my @lockTimes =
               sort { $a <=> $b }
-              map {
-                $_ =~ s/\D//;
-                abs $_;
-              }
+              map  { abs(s/\D//r) }
               grep { /\d+/ }
               split /\s*,\s*/, $conf->{bruteForceProtectionLockTimes} || '';
             $conf->{bruteForceProtectionLockTimes} = join ', ', @lockTimes
@@ -932,6 +953,20 @@ sub tests {
             return ( $res, join( ', ', @msg ) );
         },
 
+        oidcIssuerNotEnabled => sub {
+            if ( keys %{ $conf->{oidcRPMetaDataOptions} || {} } ) {
+                if ( $conf->{issuerDBOpenIDConnectActivation} ) {
+                    return 1;
+                }
+                else {
+                    return ( 1,
+"OIDC relying parties require enabling the OpenID Connect Issuer in General Parameters"
+                    );
+                }
+            }
+            return 1;
+        },
+
         # CAS APP URL must be defined and unique
         casAppHostnameUniqueness => sub {
             return 1
@@ -971,6 +1006,20 @@ sub tests {
             return ( $res, join( ', ', @msg ) );
         },
 
+        casIssuerNotEnabled => sub {
+            if ( keys %{ $conf->{casAppMetaDataOptions} || {} } ) {
+                if ( $conf->{issuerDBCASActivation} ) {
+                    return 1;
+                }
+                else {
+                    return ( 1,
+"CAS applications require enabling the CAS Issuer in General Parameters"
+                    );
+                }
+            }
+            return 1;
+        },
+
         # Notification system required with removed SF notification
         sfRemovedNotification => sub {
             return 1 unless ( $conf->{sfRemovedMsgRule} );
@@ -1076,6 +1125,18 @@ sub tests {
             return 1;
         },
 
+        # When auth is choice, userDB should be choice also
+        userdbChoice => sub {
+            return 1
+              unless ( $conf->{authChoiceModules}
+                and %{ $conf->{authChoiceModules} }
+                and $conf->{authentication} eq 'Choice' );
+            return ( 1, 'UserDB should be Same when authentication is Choice' )
+              unless $conf->{userDB} eq 'Same'
+              or $conf->{userDB} eq 'Choice';
+            return 1;
+        },
+
         # FindUser authChoice parameter must be defined
         findUserChoiceParam => sub {
             return ( -1, 'FindUser choice parameter must be defined' )
@@ -1143,7 +1204,74 @@ sub tests {
                 }
             }
             return 1;
-        }
+        },
+
+        # OIDC Signature and Encryption tests
+        oidcSigAlgShouldMatchKeyType => sub {
+            for my $key (
+                qw(oidcRPMetaDataOptionsIDTokenSignAlg oidcRPMetaDataOptionsAccessTokenSignAlg oidcRPMetaDataOptionsUserInfoSignAlg)
+              )
+            {
+                foreach my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} } ) {
+                    return ( 0,
+"Signature algorithm shouldn't be ES* if key type is RSA ($rp/$key)"
+                      )
+                      if $conf->{oidcRPMetaDataOptions}->{$rp}->{$key}
+                      and $conf->{oidcRPMetaDataOptions}->{$rp}->{$key} =~ /^E/
+                      and $conf->{oidcServiceKeyTypeSig} ne 'EC';
+                    return ( 0,
+"Signature algorithm shouldn't be RS* or PS* if key type is EC ($rp/$key)"
+                      )
+                      if $conf->{oidcRPMetaDataOptions}->{$rp}->{$key}
+                      and $conf->{oidcRPMetaDataOptions}->{$rp}->{$key} !~ /^E/
+                      and $conf->{oidcServiceKeyTypeSig} eq 'EC';
+                }
+            }
+            return 1;
+        },
+
+# Warn if both oidcRPMetaDataOptionsJwks and oidcRPMetaDataOptionsJwksUri is set
+        noJwksDuplication => sub {
+            return 1
+              unless $conf->{oidcRPMetaDataOptions}
+              and ref $conf->{oidcRPMetaDataOptions};
+            my @pb;
+            for my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} } ) {
+                push @pb, $rp
+                  if $conf->{oidcRPMetaDataOptions}->{$rp}
+                  ->{oidcRPMetaDataOptionsJwks}
+                  and $conf->{oidcRPMetaDataOptions}->{$rp}
+                  ->{oidcRPMetaDataOptionsJwksUri};
+            }
+            return 1 unless @pb;
+            return ( 1,
+                "JWKS URI defined while JWKS document is fixed: "
+                  . join( ', ', @pb ) );
+        },
+
+# If oidcRPMetaDataOptionsAuthRequiredForAuthorize is set, oidcRPMetaDataOptionsAuthMethod should be compatible
+        oidcCompatAuth => sub {
+            return 1
+              unless $conf->{oidcRPMetaDataOptions}
+              and ref $conf->{oidcRPMetaDataOptions};
+            my @pb;
+            for my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} } ) {
+                push @pb, $rp
+                  if $conf->{oidcRPMetaDataOptions}->{$rp}
+                  ->{oidcRPMetaDataOptionsAuthRequiredForAuthorize}
+                  and $conf->{oidcRPMetaDataOptions}->{$rp}
+                  ->{oidcRPMetaDataOptionsAuthMethod}
+                  and $conf->{oidcRPMetaDataOptions}->{$rp}
+                  ->{oidcRPMetaDataOptionsAuthMethod} !~
+                  /^(?:client_secret|private_key)_jwt$/;
+            }
+            return 1 unless @pb;
+            return ( 1,
+                    'Incompatible required authentication methods in RP '
+                  . '(only client_secret_jwt and private_key_jwt are allowed '
+                  . 'when authentication is required on authorization endpoint: '
+                  . join( ', ', @pb ) );
+        },
     };
 }
 
